#  Copyright (C) 2010 Cardapio Team (tvst@hotmail.com)
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

try:
	import sys,dbus, dbus.service, gi
	gi.require_version('PanelApplet', '5.0')
	gi.require_version('Gtk', '3.0')
	from gi.repository import Gtk, Gio, PanelApplet
	from dbus.mainloop.glib import DBusGMainLoop
	from Misc import *
except Exception, exception:
	fatal_error("Loading Cardapio's applet for the Gnome Panel", exception)
	sys.exit(1)

class CardapioGnomeApplet():
	popupmenu_xml = """<section> <item> <attribute name="label" translatable="no">Preferences</attribute> <attribute name="action">cardapio.preferences</attribute> </item> </section>"""

	''' Create a new CardapioApplet instance '''
	def __init__(self, applet):
		settings = Gtk.Settings.get_default()
		settings.set_property('gtk-button-images', True)
		self.applet = applet
		self.button = Gtk.ToggleButton()
		self.button.set_relief(Gtk.ReliefStyle.NONE)
		style_context = self.button.get_style_context()
		css_provider = Gtk.CssProvider()
		css = "GtkButton { padding: 0px 0px; } "
		css_provider.load_from_data(css)
		style_context.add_provider(css_provider,Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
		self.changing = False

		self.applet.set_background_widget(self.button)
		self.applet.add(self.button)

		# Create dbus connection
		bus = dbus.SessionBus()
		cardapio = bus.get_object("org.varal.Cardapio", "/org/varal/Cardapio")
		self.cardapio = cardapio

		# dbus call to "is visible" method
		self.is_cardapio_visible = cardapio.get_dbus_method("is_visible")

		# The show/hide method
		self._click_handler_id = self.button.connect("clicked", self.show_near_point)

		# If you have no handle you need to capture the button-press-event and re-emit it
		# this makes sure the context menu works
		self.button.connect("button-press-event", self.click_handler)

		# dbus call to "cardapio preferences"
		self.show_prefs = cardapio.get_dbus_method("open_options_dialog")

		def visibility_toggled(data=None):
			# if cardapio hides, button = not toggled. etc.
			visibility = self.is_cardapio_visible()
			self.button.handler_block(self._click_handler_id)
			self.button.set_active(visibility)
			self.button.handler_unblock(self._click_handler_id)
		cardapio.connect_to_signal("on_menu_visibility_changed", visibility_toggled)

		# Loaded! Set up!
		def we_loaded(data=None):
			preferences = self.get_preferences()
			self.button.set_label(preferences[0])
			self.set_applet_icon(preferences[1])

		# When cardapio is loaded, set ourselves up accordingly
		cardapio.connect_to_signal("on_cardapio_loaded", we_loaded)

		# dbus call to "show/hide" method
		self.show_hide = cardapio.get_dbus_method("show_hide_near_point")

		# dbus call to "quit" method
		self.quitty = cardapio.get_dbus_method("quit")

		# Setup the menu's action group
		group = Gio.SimpleActionGroup.new()

		# Add preferences item to menu
		action = Gio.SimpleAction.new("preferences", None)
		action.connect("activate", self.on_preferences)
		group.add_action(action)

		# Create the menu (TODO: gettext translation)
		self.applet.setup_menu(self.popupmenu_xml, group, "")

		self.applet.insert_action_group("cardapio", group);

		# If you want a "handle" use this :)
		#applet.set_flags(PanelApplet.AppletFlags.HAS_HANDLE)
		applet.set_flags(PanelApplet.AppletFlags.EXPAND_MINOR)

		# Hook up a preferences method
		self.get_preferences = cardapio.get_dbus_method("get_applet_configuration")

		# Show applet
		self.applet.show_all()

		# Hook destroy/quit
		self.applet.connect("destroy", self.shutusdown)

	# Set the applets icon
	def set_applet_icon(self, icon):
		image = Gtk.Image()
		if "." in icon:
			# try as a filename
			try:
				image.set_from_file(icon)
				self.button.set_property("image", image)
				print("Image is a filename:%s" % icon)
			except:
				pass
		else:
			# Try as an icon name
			try:
				image.set_from_icon_name(icon, Gtk.IconSize.LARGE_TOOLBAR - 2)
				self.button.set_property("image", image)
				print("Image is a 22px icon name:%s" % icon)
			except:
				try:
					image.set_from_icon_name(icon, Gtk.IconSize.LARGE_TOOLBAR)
					self.button.set_property("image", image)
					print("Image is a 24px sized icon name:%s" & icon)
				except:
					try:
						image.set_from_icon_name("distributor-logo", Gtk.IconSize.LARGE_TOOLBAR)
						self.button.set_property("image", image)
						print("Image is a 24 sized fallback distribution logo:%s" % icon)
					except:
						print("Image %s load failed, setting text as fallback:%s" % icon, "Menu")
						self.button.set_label("Menu")
						pass

	# Quilt handler
	def shutusdown(self, wid, data=None):
		self.quitty()
		Gtk.main_quit()

	# Click handler
	def click_handler(self, widg, event):
		if event.button == 3:
			# ignore right clicks, so the applet has a context menu
			widg.emit_stop_by_name('button-press-event')
		return False

	# Show main UI
	def show_near_point(self, w,data=None):
		# find out where we are on screen ^^
		window = self.applet.get_window()
		xy = window.get_origin()
		x_point = xy[1]
		y_point = xy[2]
		self.show_hide(x_point,y_point)

	# Hook main UI's preferences window
	def on_preferences(self, action, parameter):
		print("Calling preferences")
		self.show_prefs()

# Entry point
def CardapioGnomeAppletFactory(applet, iid, data = None):
	print('Starting cardapio factory instance: %s %s %s'% ( applet, iid, data ))
	DBusGMainLoop(set_as_default=True)
	the_applet = CardapioGnomeApplet(applet)
	return True
